!> \file step.F90
!! TOOOOOOOOO DOOOOOOOOOOOOO
!! Step_grisli contient
!<


subroutine step_grisli()

  use module3d_phy
  use module_choix ! module de choix du type de run
  !  module_choix donne acces a tous les modules
  !  de declaration des packages
  use interface_icetempmod
  use sorties_ncdf_grisli
  use flottab_mod
  use diagno_mod
  use resolmeca_SIA_L1
  use track_debug 

  use geom_type
  use temperature_type
  use ice_flow_type
  use mask_flgz_type
  use deformation_type
  use autre_pr_temp_type
  use step_temp_type
  
  
implicit none

  integer :: m
  logical :: shelf_vitbil = .true.   ! si vrai on prend les vitesses de bilan pour le shelf

!!$if (time.lt.timemax-100) then
!!$   print*,'le modele remonte le temps de',timemax,'a',time
!!$   stop
!!$endif
  timemax=time

  if (itracebug.eq.1) write(num_tracebug,*) 'entree dans step_grisli', &
       '   nt=',nt,'iter_beta=',iter_beta
  !--------------------------------------------------------------------
  !     lecture dans un petit fichier au cas ou les parametres du run  
  !     seraient changes                                               
  !--------------------------------------------------------------------

  call limit_file(nt,real(time),dt,tend,dtsortie,dtcpt,testdiag,dtt,runname)

  if (time.lt.tgrounded) then
     marine=.false.
  else
     marine=.true.
  endif

  !if ((iter_beta.eq.0).or.(nt.le.1)) then
  call masque()
  call flottab()
  if (iter_beta.gt.0)   call init_dragging
  !end if


  !-----------------------------------------------------------------------!
  !              debut bloc appels tous les dtt                           !
  !-------------   -------------------------------------------------------!


  isync_1: if (ISYNCHRO.eq.1) then          !debut bloc appels tous les dtt    



     spinup_1_veloc: if (ispinup.ne.2.or.nt.lt.5) then   

     !---------------------------------------------------------------------------------
     ! spinup_1 : calcul des vitesses dynamiques :
     !------------
     !            - est fait dans tous les cas pour les premieres boucles temporelles
     !            - est fait la plupart des types de spinup sauf ispinup=2
     !            - ispinup=1 ->  modif: maintenant on fait le calcul de veloc
     !---------------------------------------------------------------------------------

        if (itracebug.eq.1)  call tracebug(' Dans spinup_1_veloc : calculer velocities')

        call forclim()                                  !
        call ablation

        call Neffect()                                             
        call velocities()                                            
     endif spinup_1_veloc

     if (nt.gt.2) then
        ! Hassine ca va etre modifier bien sur apres: l appel sera beaucoup moin long que ca 

        call Allocate_geom(geom_g,nx,ny)
        call Init_geom(Geom_g)               
          
        call Allocate_temperature(temperature_g,nx,ny,nz,nz+nzm)
        call Init_Temperature(Temperature_g)

        call Allocate_Ice_flow(Ice_flow_g,nx,ny,nz)
        call Init_Ice_flow(Ice_flow_g)
       
        call Allocate_Mask_flgz(Mask_flgz_g,nx,ny)
        call Init_Mask_flgz(Mask_flgz_g)
        
        call Allocate_Deformation(Deformation_g,nx,ny,nz,n1poly,n2poly)
        call Init_Deformation_h(Deformation_g)
        
        call Allocate_autre_pr_temp(Autre_pr_temp_g,nx,ny)
        call Init_autre_pr_temp(Autre_pr_temp_g)
     
        call Init_step_temp(step_temp_g)
        
        call icetemp(geom_g,temperature_g, &
             Ice_flow_g,Mask_flgz_g,&
             deformation_g,Autre_pr_temp_g,&
             step_temp_g,num_tracebug)

        T= Temperature_g%Temperature 
        Tpmp= Temperature_g%Temp_melting
        Tbdot= Autre_pr_temp_g%Tbdot
        Bmelt= Autre_pr_temp_g%BMELT
        Ibase=Autre_pr_temp_g%IBASE
        Phid=Autre_pr_temp_g%Phid

        !      write(6,*) time,'call icetemp'
        ! subroutines pour le calcul de la fusion basale
        call bmeltshelf
        call bmelt_grounded


     endif

    spinup_2_flowlaw: if ((ispinup.ne.2).or.(nt.lt.5)) then

    !---------------------------------------------------------------------------------
    ! spinup_2_flowlaw
    !----------------
    ! si ispinup =   2 , on ne fait pas le calcul de flowlaw, ni de diffusiv 
    !                    puisque les vitesses de bilan sont imposees (initialisee nt<5)
    !                   
    ! Flowlaw et diffusiv : servent soit directement (ispinup = 0,1) soit pour le 
    !                       rescaling (ispinup = 3) 
    !---------------------------------------------------------------------------------

        if (itracebug.eq.1)  call tracebug(' Dans spinup_2_flowlaw')

        if (ispinup.eq.3) then
           call diffusiv                  ! pour vrifier les champs
           debug_3D(:,:,71)=ux(:,:,1)
           debug_3D(:,:,72)=uy(:,:,1)
           debug_3D(:,:,13)=ux(:,:,nz)
           debug_3D(:,:,14)=uy(:,:,nz)
        end if

        call flow_general
        do iglen=n1poly,n2poly
           call flowlaw(iglen)
        end do


!        debug_3D(:,:,65)=s2a_mx(:,:,1,1)
!        debug_3D(:,:,66)=s2a_mx(:,:,1,2)
!        debug_3D(:,:,67)=s2a_my(:,:,1,1)
!        debug_3D(:,:,68)=s2a_my(:,:,1,2)
!        debug_3D(:,:,69)=ddx(:,:,1)
!        debug_3D(:,:,69)=ddx(:,:,2)

        if (ispinup.eq.3) then
           call calc_coef_vitbil                    ! recalibre sa_mx et s2a_mx et ddbx
           !         call limit_coef_vitbil         ! garde la loi de deformation (debug)
        end if

     end if spinup_2_flowlaw

  endif isync_1


  spinup_3_bed: if (ispinup.eq.0.or.nt.lt.5) then
    !---------------------------------------------------------------------------------
    ! spinup_3_bed
    !----------------
    ! Ne passe dans ce bloc que quand on veut l'isostasie, donc si variations epaisseur
    ! run standard  (ispinup=0) et pour les initialisations                  
    !---------------------------------------------------------------------------------

     if (itracebug.eq.1)  call tracebug(' Dans spinup_3_bed')

     if ((nbed.eq.1).and.nt.ne.1.and.isynchro.eq.1) then           
        call bedrock()                                            !  bedrock adjustment
     endif
  end if spinup_3_bed

 spinup_4_vitdyn: if  (ispinup.ge.1.or.nt.lt.5.) then  ATTENTION A MODIFIER ipsinup=3  shelf_vitbil

    !---------------------------------------------------------------------------------
    ! spinup_4_vitdyn
    !----------------
    ! Ne passe dans ce bloc que quand on veut calculer les vitesses dynamiques :
    ! - initialisations (nt.lt.5)
    ! - run standard  (ispinup=0) (y compris iter_beta.ne.0)
    ! - spinup temperature avec vitesses dynamiques  (ispinup = 1)                
    !---------------------------------------------------------------------------------

     !     (*********** DIFFUSIVITIES - VITESSES SHELF **********************)


     call Neffect()

     call diffusiv()


iterbeta:     if (iter_beta.ne.0) then          ! pour les iterations sur beta

        if (.not.shelf_vitbil) then    !  bloc si les vitesses shelves viennent de diagnoshelf

                                       !  Vcol declare dans le module spinup_mod 
                                       !  ou dans le dragging si no_spinup

           where (flotmx(:,:))
              uxbar(:,:)=uxflgz(:,:)
           elsewhere
              uxbar(:,:)=Vcol_x(:,:)
           end where

           where (flotmy(:,:))
              uybar(:,:)=uyflgz(:,:)
           elsewhere
              uybar(:,:)=Vcol_y(:,:)
           end where
        else
           uxbar(:,:)=Vcol_x(:,:)
           uybar(:,:)=Vcol_y(:,:)
        end if
     end if iterbeta


     call strain_rate()
 
    if (iter_beta.ne.0) then
        uxbar(:,:)=uxflgz(:,:)+uxdef(:,:)
        uybar(:,:)=uyflgz(:,:)+uydef(:,:)
     end if



     !-----------------------------------------------------------------------
     !              debut bloc appels tous les dtt                         
     !-------------   -------------------------------------------------------

     if ((shelfy).and.(marine).and.(isynchro.eq.1)) then                

        call lake_to_slv
        !write(6,*) 'premier diagno dans step'
        !call flottab

        ! les 3 lignes ci-dessous pour avoir les champs avant diagno (en cas de pb colineaire)
        !call init_sortie_ncdf
        !call sortie_ncdf_cat

        call diagnoshelf

        !call sortie_ncdf_cat
        ! STOP                  


        !         faire 10 tours pour equilibrer                                
        if ((icompteur.eq.0).and.(nt.lt.10)) then                       
           do m=1,1                                    
              !         write(6,*) 'dans boucle m'
              call diagnoshelf()                                        
           end do
        end if

     endif !

     call  mix_SIA_L1

     debug_3D(:,:,78)= abs(uxbar(:,:))-abs(Vcol_x(:,:))
     debug_3D(:,:,79)= abs(uybar(:,:))-abs(Vcol_y(:,:))

     debug_3D(:,:,80)= abs(uxdef(:,:))-abs(Vcol_x(:,:))
     debug_3D(:,:,81)= abs(uydef(:,:))-abs(Vcol_y(:,:))


  end if spinup_3_bed

end subroutine step_grisli

!--------------------------------------------------------------------------
! step_grisli1 est a l'interieur de la boucle temps. 
! il commence par icethick, qui dtermine dt, isynchro

! ensuite contient les appels aux diverses sorties
! puis un appel a step_grisli


subroutine step_grisli1                

  use module3d_phy

  use geom_type
  use temperature_type
  use ice_flow_type
  use mask_flgz_type
  use deformation_type
  use autre_pr_temp_type

  use module_choix ! module de choix du type de run
  !  module_choix donne acces a tous les modules
  !  de declaration des packages
  use sorties_ncdf_grisli
  use flottab_mod
  use interface_icetempmod
  use diagno_mod 
  use track_debug 

  if (itracebug.eq.1) write(num_tracebug,*) ' Entree dans step_grisli1',   &
       '   nt=',nt,'iter_beta=',iter_beta

  nospinup_4: if ((ispinup.eq.0.or.nt.lt.5).and.(iter_beta.eq.0)) then
     if (itracebug.eq.1)  call tracebug(' Dans nospinup_4 : appelle icethick')

     call icethick3()

     call calving

     if (itracebug.eq.1)  call tracebug('apres calving')

  else if ((ispinup.eq.1).or.(ispinup.eq.3).or.(iter_beta.gt.0)) then
     dt=dtmax
     hdot(:,:)=0.

     !      if (nt.eq.6) time=0.
     !      time=nint(dtmax*nint(time/dtmax))+dtmax  semble poser des problemes
     time=nt*dtmax

     if (itracebug.eq.1) write(num_tracebug,*) ' Dans nospinup_4  no icethick ',ispinup, &
          '    time=',time
  else if (ispinup.eq.2) then
     call force_balance_vel
     call icethick3()
     call calving

  end if nospinup_4

  CALL write_trace_940('ICE SHEET SURFACE')

  where(.not.flot(:,:))
     S(:,:)=Bsoc(:,:)+H(:,:)
     B(:,:)=Bsoc(:,:)
  end where

  ! calcul de Hmx et Hmy -> shift=-1, dim=1 -> H(i-1,j)

  hmx(:,:)=0.5*(H(:,:)+eoshift(H(:,:),shift=-1,boundary=0.,dim=1))
  hmy(:,:)=0.5*(H(:,:)+eoshift(H(:,:),shift=-1,boundary=0.,dim=2))
  hmx(1,:)=0.
  hmy(:,1)=0.

  ! mise a 0 des vitesses quand epaisseur nulle
  where (hmx(:,:).le.1.)
     uxbar(:,:)=0.
  endwhere
  where (hmy(:,:).le.1.)
     uybar(:,:)=0.
  endwhere

  !     ________________________________  fin bloc appel tous les tend

  !     (****** ICE SHEET STATISTICS and SHORT OUTPUT *******)
  !     if (mod(nt,nint(NDISP/DT)).eq.0).or.(nt.eq.1).or.(nt.eq.2).or.
  !    &  (nt.eq.3).or.(nt.eq.4).or.(nt.eq.5).or.(nt.eq.10).or.

  !     if ((mod(abs(TIME),NDISP*1.).lt.dtmin).or.(NT.le.5).or.(NT.eq.10) &
  !            .or.(nt.eq.15).or.(nt.eq.20).or.(nt.eq.30).or. &
  !            (abs(TIME-TEND).lt.dtmin)) then

  !        call sortie_ncdf_cat


  if ((mod(abs(TIME),NDISP*1.).lt.dtmin).or.(isynchro.eq.1)) then

     call shortoutput()

     if ((geoplace.eq.'anteis1') &
          .or.(geoplace.eq.'ant20km')) then

        call ts_out()
     endif
     !        call sealevel_out

  endif


  ! Sorties horizontales.
  !--------------------------------------------------------------------------------
  !  Premiere partie des sorties horizontales

  !     if (mod(abs(dble(TIME)),dble(DTSORTIE)).lt.dble(dtmin)) then
  !

  if ((mod(abs(TIME),dtt).lt.dtmin).or.(isynchro.eq.1)) then
     isynchro=1
     !     if (isynchr o.eq.1) then
     call testsort_time(real(time))
     if (iglob_hz.eq.1) then
        call sortie_hz_multi   ! pour les variables dclares dans 3D
        !              call hz_output(real(time))
     endif

  else
     iglob_hz=0

  endif


  !   *** Sortie des profils tous les dtprofile ***
  if ((NT.eq.1) &    !   .or.(NT.eq.2)
       .or.(mod(abs(dble(TIME)),dble(DTPROFILE)).lt.dble(dtmin)) &
       .or.(abs(TIME-TEND).lt.dtmin)) then

     !        call sortieprofile()

  endif

  !     *** SORTIE COMPTEUR TOUS LES DTCPT ANS ***

  if ((mod(abs(dble(TIME)),dble(DTCPT)).lt.dble(dtmin)).OR.   &
       (ABS(TIME+297000).LT.dtmin).OR.   &
       (ABS(TIME+126000).LT.dtmin).OR.   &
       (ABS(TIME+21000).LT.dtmin).OR.    &
       (ABS(TIME+16000).LT.dtmin))    THEN

     call sortiecptr()

  endif


  !  ************* sorties netcdf ****************

  call testsort_time_ncdf(real(time))
  if (iglob_ncdf .EQ. 1) call sortie_ncdf_cat

  !   call sortie_ncdf_cat

  call track_change_T

  call step_grisli()
  !
  !
  !
end subroutine step_grisli1
